from django.contrib import admin
from django.conf import settings

from live_support.models import Chat, ChatMessage

class ChatAdmin(admin.ModelAdmin):
    list_display = ('id','name','started',)
    list_display_links = ('id','name',)
    list_filter = ('started',)
    
    class Media:
        js = (
            "live_support/js/live_support_admin.js",
            "live_support/js/live_support.js",
        )
        css = {
            'all': ('live_support/css/live_support.css',),
        }
    
    def changelist_view(self, request, extra_context=None):
        pending_chats = Chat.objects.filter(ended=None, agents=None).order_by('-started')
        active_chats = Chat.objects.filter(ended=None).filter(agents=request.user)
        c = {
            'pending_chats': pending_chats,
            'active_chats': active_chats,
        }
        print "ACTIVE CHATS: %s" % active_chats
        return super(ChatAdmin, self).changelist_view(request, extra_context=c)


admin.site.register(Chat, ChatAdmin)
admin.site.register(ChatMessage)





from django.conf.urls.defaults import patterns, include, url
from live_support import views

urlpatterns = patterns('', 
    url('^$', views.start_chat), 
    url('^ajax/get_messages/$', views.get_messages), 
    url('^ajax/(?P<chat_id>\d+)/post_message/$', views.post_message), 
    url('^ajax/(?P<chat_id>\d+)/end_chat/$', views.end_chat), 
    url('^ajax/(?P<chat_id>\d+)/join_chat/$', views.join_chat), 
    url('^(?P<chat_uuid>[\w-]+)/get_messages/$', views.client_get_messages), 
    url('^(?P<chat_uuid>[\w-]+)/post_message/$', views.client_post_message), 
    url('^(?P<chat_uuid>[\w-]+)/$', views.client_chat)
)




from django import forms

from live_support.models import Chat, ChatMessage


class ChatForm(forms.ModelForm):
    details = forms.CharField(widget=forms.Textarea, label='Question')
    class Meta:
        model = Chat
        fields = ('name','details',)

class ChatMessageForm(forms.ModelForm):
    message = forms.CharField()
    class Meta:
        model = ChatMessage
        fields = ('message',)



from django import template
from django.conf import settings
from datetime import timedelta, datetime
from django.core.urlresolvers import reverse

from live_support.models import Chat

register = template.Library()

def chat_iframe(context):
    request = context['request']
    # The default url is the Start Chat page
    iframe_url = reverse('live_support.views.start_chat')
    if request.session.get('chat_hash_key'):
        chat = Chat.objects.filter(hash_key=request.session['chat_hash_key'])
        if chat and not chat[0].ended:
            # If the user currently has an active chat session and it has not
            # ended, display that instead.
            iframe_url = reverse('live_support.views.client_chat', args=[chat[0].hash_key,])

    return {
        'STATIC_URL': settings.STATIC_URL,
        'url': iframe_url,
    }

register.inclusion_tag('live_support/chat_iframe.html', takes_context=True)(chat_iframe)




from django.conf.urls.defaults import patterns, include, url
from django.contrib import admin
from django.views.generic.simple import direct_to_template

admin.autodiscover()

urlpatterns = patterns('', 
    url('^admin/', include(admin.site.urls)), 
    url('^chat/$', direct_to_template, {
        'template': 'chat.html'
    }),
    url('^', include('live_support.urls')),
)




# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Chat'
        db.create_table('live_support_chat', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('hash_key', self.gf('django.db.models.fields.CharField')(unique=True, max_length=64, blank=True)),
            ('details', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('started', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('ended', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
        ))
        db.send_create_signal('live_support', ['Chat'])

        # Adding M2M table for field agents on 'Chat'
        db.create_table('live_support_chat_agents', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('chat', models.ForeignKey(orm['live_support.chat'], null=False)),
            ('user', models.ForeignKey(orm['auth.user'], null=False))
        ))
        db.create_unique('live_support_chat_agents', ['chat_id', 'user_id'])

        # Adding model 'ChatMessage'
        db.create_table('live_support_chatmessage', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('chat', self.gf('django.db.models.fields.related.ForeignKey')(related_name='messages', to=orm['live_support.Chat'])),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255, blank=True)),
            ('agent', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'], null=True, blank=True)),
            ('message', self.gf('django.db.models.fields.TextField')()),
            ('sent', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal('live_support', ['ChatMessage'])


    def backwards(self, orm):
        # Deleting model 'Chat'
        db.delete_table('live_support_chat')

        # Removing M2M table for field agents on 'Chat'
        db.delete_table('live_support_chat_agents')

        # Deleting model 'ChatMessage'
        db.delete_table('live_support_chatmessage')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'live_support.chat': {
            'Meta': {'object_name': 'Chat'},
            'agents': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'chats'", 'blank': 'True', 'to': "orm['auth.User']"}),
            'details': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'ended': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'hash_key': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '64', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'started': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'})
        },
        'live_support.chatmessage': {
            'Meta': {'object_name': 'ChatMessage'},
            'agent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'}),
            'chat': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'messages'", 'to': "orm['live_support.Chat']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'message': ('django.db.models.fields.TextField', [], {}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'sent': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['live_support']



import os
from django.conf import settings
from django.db import models
from django.contrib.auth.models import User
from datetime import datetime
from uuid import uuid4 

class ChatManager(models.Manager):   
    def get_query_set(self):
        return super(ChatManager, self).get_query_set().filter(ended=None)

class Chat(models.Model):
    name = models.CharField(max_length=255)
    hash_key = models.CharField(unique=True, max_length=64, 
            editable=False, blank=True, default=uuid4)
    details = models.TextField(blank=True)
    started = models.DateTimeField(auto_now_add=True)
    ended = models.DateTimeField(null=True, blank=True)
    agents = models.ManyToManyField(User, blank=True, related_name='chats')
    objects = models.Manager()
    active = ChatManager()

    def __unicode__(self):
        return '%s: %s' % (self.started, self.name)

    def end(self):
        self.ended = datetime.now()
        self.save()
    
    class Meta:
            permissions = (
                ("chat_admin", "Chat Admin"),
            )

class ChatMessage(models.Model):
    chat = models.ForeignKey(Chat, related_name='messages')
    name = models.CharField(max_length=255, blank=True)
    agent = models.ForeignKey(User, blank=True, null=True)
    message = models.TextField()
    sent = models.DateTimeField(auto_now_add=True)

    def get_name(self):
        if self.agent:
            return self.agent.first_name or self.agent.username
        else:
            return self.name

    def __unicode__(self):
        return '%s: %s' % (self.sent, self.message)




import json
from django.db import models 
from django.core.urlresolvers import reverse
from django.http import HttpResponse, Http404, HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.core import serializers
from django.contrib.auth.decorators import permission_required, user_passes_test
from django.core.cache import cache
from django.utils.html import escape

from live_support.models import Chat, ChatMessage
from live_support.forms import ChatMessageForm, ChatForm

@permission_required('live_support.chat_admin')
def join_chat(request, chat_id):
    chat = get_object_or_404(Chat, id=chat_id)
    if request.user.is_authenticated():
        chat.agents.add(request.user)
    return HttpResponseRedirect(request.META['HTTP_REFERER'])

@permission_required('live_support.chat_admin')
def post_message(request, chat_id):
    chat = get_object_or_404(Chat, id=chat_id)
    last_message_id = request.POST.get('last_message_id')
    message_form = ChatMessageForm(request.POST or None)
    if message_form.is_valid():
        message = message_form.save(commit=False)
        message.chat = chat
        message.agent = request.user
        message.name = message.get_name()
        message.save()
    if request.is_ajax():
        if last_message_id:
            new_messages = chat.messages.filter(id__gt=last_message_id)
        else:
            new_messages = chat.messages.all()
        new_message_list = []
        for message in new_messages:
            new_message_list.append({
                'name': escape(message.name),
                'message': escape(message.message),
                'pk': message.pk,
                'chat': chat.id,
            })
        return HttpResponse(json.dumps(new_message_list))
    return HttpResponseRedirect(request.META['HTTP_REFERER'])
    
@permission_required('live_support.chat_admin')
def end_chat(request, chat_id):
    chat = get_object_or_404(Chat, id=chat_id)
    if request.POST.get('end_chat') == 'true':
        chat.end()
    return HttpResponseRedirect(request.META['HTTP_REFERER'])

@permission_required('live_support.chat_admin')
def get_messages(request):
    """
        For each chat id passed in via querystring, get the last message id and
        query for any new messages for that chat session. Also include all 
        pending chat sessions.
    """
    chats = {}
    for k, v in request.GET.iteritems():
        alive = True
        messages = ChatMessage.objects.filter(chat__id=k)
        if v:
            messages = ChatMessage.objects.filter(chat__id=k, id__gt=v)

        # Check to see if the end user has recently checked for new messages
        # for this chat session by checking for the cache entry using the 
        # chat id.  If they haven't asked for new messages in the past 30
        # seconds they probably navigated away or closed the window.
        if not cache.get('chat %s' % k):
            alive = False
        
        message_list = []
        for message in messages:
            message_list.append({
                'name': escape(message.name),
                'message': escape(message.message),
                'pk': message.pk,
            })
        chats[k] = {
            'messages': message_list,
            'alive': alive
        }
    
    # Get the list of pending chat sessions, and for each one get a url for 
    # joining that chat session.
    pending_chats = list(Chat.objects.filter(ended=None).exclude(agents=request.user).order_by('-started'))
    pending_chats_list = [{
        'name': escape(chat.name), 
        'url': reverse('live_support.views.join_chat', args=[chat.id]),
    } for chat in pending_chats ]

    output = {
        'chats': chats,
        'pending_chats': pending_chats_list,
    }
    
    # Dump the whole thing to json and return it to the browser.
    return HttpResponse(json.dumps(output))


def client_get_messages(request, chat_uuid):
    """
        Get any new chat messages for the chat current chat session (based on 
        hash_key) and return them to the browser as json.
    """
    chat = get_object_or_404(Chat, hash_key=chat_uuid)
    cache.set('chat %s' % chat.id, 'active', 20)
    last_message_id = request.GET.get(str(chat.id))
    if last_message_id:
        messages = chat.messages.filter(id__gt=last_message_id)
    else:
        messages = chat.messages.all()

    message_list = []
    for message in messages:
        message_list.append({
            'name': escape(message.name),
            'message': escape(message.message),
            'pk': message.pk,
        })
    chats = { 
        chat.id: {
            'messages': message_list, 
        }
    }
    output = {
        'chats': chats,
        'pending_chats': [],
    }
    return HttpResponse(json.dumps(output))

def client_post_message(request, chat_uuid):
    """
        Post the message from the end user and return a list of any
        new messages based on the last_messag_id specified.
    """
    chat = get_object_or_404(Chat, hash_key=chat_uuid)
    last_message_id = request.POST.get('last_message_id')
    message_form = ChatMessageForm(request.POST or None)
    if message_form.is_valid():
        message = message_form.save(commit=False)
        message.chat = chat
        message.name = chat.name
        message.save()
    if last_message_id:
        new_messages = chat.messages.filter(id__gt=last_message_id)
    else:
        new_messages = chat.messages.all()
    new_message_list = []
    for message in new_messages:
        new_message_list.append({
            'name': escape(message.name),
            'message': escape(message.message),
            'pk': message.pk,
            'chat': chat.id,
        })
    return HttpResponse(json.dumps(new_message_list))

def client_chat(request, chat_uuid):
    chat = get_object_or_404(Chat, hash_key=chat_uuid)
    message_form = ChatMessageForm(request.POST or None)
    if message_form.is_valid():
        message = message_form.save(commit=False)
        message.chat = chat
        message.name = chat.name
        message.save()
        message_form = ChatMessageForm()

    params = {
        'chat': chat,
        'message_form': message_form,
    }
    return render_to_response('live_support/live_support.html', params, context_instance=RequestContext(request))

def start_chat(request):
    chat_form = ChatForm(request.POST or None)
    if chat_form.is_valid():
        chat = chat_form.save()
        request.session['chat_hash_key'] = chat.hash_key
        return HttpResponseRedirect(reverse('live_support.views.client_chat', args=[chat.hash_key,]))

    params = {
        'chat_form': chat_form,
    }
    return render_to_response('live_support/start_chat.html', params, context_instance=RequestContext(request))
